package main

import (
	"flag"
	"html/template"
	"io/ioutil"
	"log"
	"net/http"
	"os"
	"path"
	"strconv"
	"strings"

	"github.com/russross/blackfriday"
)

func main() {
	loadConfig()

	if err := updateTemplate(); err != nil {
		log.Fatal(`Failed to load template. (` + err.Error() + `)`)
	}

	log.Fatal(http.ListenAndServe(":"+strconv.Itoa(config.Port), http.HandlerFunc(serveRequest)))
}

var config struct {
	Port     int
	Template string
	Pages    string
	Static   string
}

func loadConfig() {
	flag.IntVar(&config.Port, `port`, 80, `The port mdsite will listen on`)
	flag.StringVar(&config.Template, `template`, `template.html`, `The template used by mdsite`)
	flag.StringVar(&config.Pages, `pages`, `pages`, `The OS path used when searching a page`)
	flag.StringVar(&config.Static, `static`, `static`, `The OS path used for static resource`)
	flag.Parse()
}

var t *template.Template

func updateTemplate() error {
	var err error
	t, err = template.ParseFiles(config.Template)
	if err != nil {
		return err
	}
	return nil
}

func serveRequest(w http.ResponseWriter, r *http.Request) {
	if strings.HasPrefix(r.URL.Path, `/static/`) {
		path := path.Join(config.Static, path.Clean(strings.TrimPrefix(r.URL.Path, `/static/`)))
		http.ServeFile(w, r, path)
		return
	}
	servePage(w, r)
}

func servePage(w http.ResponseWriter, r *http.Request) {
	p := path.Join(config.Pages, path.Clean(r.URL.Path))

	s, err := os.Stat(p)
	if err == nil && s.IsDir() {
		if p[len(p)-1] != '/' {
			p += `/`
		}
		p += `index`
	}

	c, err := ioutil.ReadFile(p + `.md`)
	if err != nil {
		w.WriteHeader(404)
		return
	}

	t.Execute(w, template.HTML(blackfriday.MarkdownCommon(c)))
}
